<?php

namespace Netzmacher\Radialsearch\Eid;

use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2019 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * The class Session bundles methods for the session management
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  radialsearch
 *
 * @version 8.8.1
 * @since 8.8.1
 */
class NearestLocation extends \TYPO3\CMS\Frontend\Plugin\AbstractPlugin
{

	/**
	 * @var $_TSFE \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController
	 */
	private $_TSFE;

	/**
	 * _SQLQuery( ) :
	 *
	 * @param string $query
	 * @param string $method
	 * @param integer $line
	 * @return void
	 * @access private
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _RowFromSQLQuery( $query )
	{
		$res = $GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->_SQLError( $query, __METHOD__, __LINE__ );
		$row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_assoc( $res );
		return $row;
	}

	/**
	 * _SQLError( ) :
	 *
	 * @param string $query
	 * @param string $method
	 * @param integer $line
	 * @return void
	 * @access private
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _SQLError( $query, $method = __METHOD__, $line = __LINE__ )
	{
		$sqlError = $GLOBALS[ 'TYPO3_DB' ]->sql_error();
		if( empty( $sqlError ) )
		{
			return;
		}

		$sqlErrno = $GLOBALS[ 'TYPO3_DB' ]->sql_errno();
		$header = 'SQL-ERROR';
		$text = ''
						. '<p>'
						. '  Error: ' . $sqlError . '<br />'
						. '  Error number: ' . $sqlErrno . '<br />'
						. '  Query: ' . $query . '<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, $method, $line );
	}

	/**
	 * _clearCache( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.8.1
	 * @since 8.8.1
	 */
	private function _clearCache()
	{
		$query = 'TRUNCATE `cf_cache_pages`';
		$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->_SQLError( $query, __METHOD__, __LINE__ );

		$query = 'TRUNCATE `cf_cache_pagesection`';
		$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->_SQLError( $query, __METHOD__, __LINE__ );

		//var_dump( __METHOD__, __LINE__ );
//	Code from below is proper only, if an admin is logged in in the backend.
//		$tce = GeneralUtility::makeInstance( DataHandler::class );
//		$tce->stripslashes_values = false;
//		$tce->admin = 1;
//		$tce->start( [ ], [ ] );
//		$tce->clear_cacheCmd( $cacheCmd );
	}

	/**
	 * _initTSFE( ): 
	 *
	 * @return	void
	 * @version 8.8.0
	 * @since 8.8.0
	 */
	private function _initTSFE()
	{
		global $TYPO3_CONF_VARS;
		/**
		 * @var $TSFE \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController
		 */
		$this->_TSFE = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( 'TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController', $TYPO3_CONF_VARS, 0, 0 );
		\TYPO3\CMS\Frontend\Utility\EidUtility::initLanguage();

		// Get FE User Information
		$this->_TSFE->initFEuser();
	}

	/**
	 * _sessionStoreData:
	 *
	 * @return void
	 * @internal #t1694
	 * @version 8.8.0
	 * @since 8.8.0
	 */
	private function _sessionStoreData( $uid )
	{
		$space = $this->_sessionDataSpace();

		$arrSessionData = [
				'userPosition' => [
						'lat' => GeneralUtility::_GP( 'lat' ),
						'lon' => GeneralUtility::_GP( 'lon' ),
				],
		];

		$table = GeneralUtility::_GP( 'from' );
		if( !empty( $table ) )
		{
			$arrSessionData = $arrSessionData + [
					'nearestLocation' => [
							$table => $uid,
					],
			];
		}
		//var_dump( __METHOD__, __LINE__, $arrSessionData );

		$this->_initTSFE();
		$this->_TSFE->fe_user->setKey( $space, 'tx_radialsearch_pi1', $arrSessionData );
		$this->_TSFE->storeSessionData();

		$arrSessionData = $this->_TSFE->fe_user->getKey( $space, 'tx_radialsearch_pi1' );
		//var_dump( __METHOD__, __LINE__, $arrSessionData);
		// For development only
		//		$lat = GeneralUtility::_GP( 'lat' );
		//		$lon = GeneralUtility::_GP( 'lon' );
		//		echo __METHOD__ . ' (' . __LINE__ . '): lat ' . $lat . ', lon ' . $lon;
	}

	/**
	 * _sessionDataSpace( ): Get the name of the session data space.
	 *                        The name is user, if a frontend user is logged in.
	 *                        The name is ses, if any frontend user isn't logged in.
	 *
	 * @return	string		user || ses
	 * @version 8.8.0
	 * @since 8.8.0
	 */
	private function _sessionDataSpace()
	{
		switch( $GLOBALS[ 'TSFE' ]->loginUser )
		{
			case( true ):
				return 'user';
			default:
				return 'ses';
		}
	}

	/**
	 * _setQueryForUid
	 *
	 * @return string	$query
	 * @access private
	 * @version 8.8.1
	 * @since 8.8.1
	 */
	private function _setQueryForUid()
	{

		$from = GeneralUtility::_GP( 'from' );
		$fieldlat = GeneralUtility::_GP( 'fieldlat' );
		$fieldlocation = GeneralUtility::_GP( 'fieldlocation' );
		$fieldlon = GeneralUtility::_GP( 'fieldlon' );
		$fieldtitle = GeneralUtility::_GP( 'fieldtitle' );
		$lat = GeneralUtility::_GP( 'lat' );
		$lon = GeneralUtility::_GP( 'lon' );
		$query = "
SELECT	uid, " . $fieldtitle . ", " . $fieldlocation . ",
				ACOS(
					SIN( RADIANS( " . $fieldlat . " )) * SIN( RADIANS( " . $lat . " ))
					 + COS( RADIANS( " . $fieldlat . " )) * COS( RADIANS( " . $lat . " ))
					 * COS( RADIANS( " . $fieldlon . " ) - RADIANS( " . $lon . " ))
				)
				* 6378.2 AS `tmpdistance`
FROM " . $from . "
ORDER BY `tmpdistance`
LIMIT 0,1;
";
		return $query;
	}

	/**
	 * _setUidIfIsTable:
	 *
	 * @return integer
	 * @version 8.8.1
	 * @since 8.8.1
	 */
	public function _setUidIfIsTable()
	{
		$from = GeneralUtility::_GP( 'from' );

		if( empty( $from ) )
		{
			return;
		}

		$query = $this->_setQueryForUid();
		$row = $this->_RowFromSQLQuery( $query );
		//var_dump( __METHOD__, __LINE__, $row);
		$uid = $row[ 'uid' ];

		if( !$debug )
		{
			return $uid;
		}

		$csvParts = implode( ', ', $row );
		echo $csvParts;
		return $uid;
	}

	/**
	 * _zzDieWiPrompt( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _zzDieWiPrompt( $header, $text, $method, $line )
	{
		$prompt = '
      <h1 style="color:red;">
        ' . $header . '
      </h1>
      ' . $text . '
      <p>
        Sorry for the trouble. Browser - TYPO3 without PHP<br />
        Error occures here: ' . $method . ' at #' . $line . '
      </p>
      <p>
        If you need any help, please visit the
        <a href="http://typo3-browser-forum.de/" target="_blank" title="TYPO3 Browser Forums. 500 TYPO3-integrators are registered.">
          TYPO3 Browser Forum &raquo;</a>
      </p>
      ';
		die( $prompt );
	}

	/**
	 * setSession:
	 *
	 * @return void
	 * @internal #t1694
	 * @version 8.8.1
	 * @since 8.8.0
	 */
	public function setSession()
	{
		$debug = GeneralUtility::_GP( 'debug' );

		$uid = $this->_setUidIfIsTable();
		$this->_sessionStoreData( $uid );

		$this->_clearCache();
	}

}
