<?php

namespace Netzmacher\Radialsearch\Gateway;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2013-2019 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Gateway of the 'radialsearch' extension.
 *
 * @author	Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package	TYPO3
 * @subpackage	radialsearch
 * @version 8.8.1
 * @since       0.0.1
 */
class SqlParts
{

	public $prefixId = 'tx_radialsearch_interface';
// same as class name
	public $scriptRelPath = 'Classes/Gateway/SqlParts.php';
// path to this script relative to the extension dir.
	public $extKey = 'radialsearch';
// [Array] Current configuration of the extension manager
	private $_extConf = null;
// [Object] Parent object
	private $_pObj = null;
// [Boolean] Prompt to DRS?
	private $_drs = null;
// [Boolean] True, if maxRadius should used
	private $_andWithMaxRadius = null;
// [Boolean] True, if sword is set. False if not.
	private $_isSword = null;
// [Array] Array with elemenst lat and lon
	private $_confFields = null;
// [Array] TypoScript configuration of the filter
	private $_confFilter = null;
// [Array] TypoScript configuration of GETPOST
	private $_confGP = null;

	/**
	 * _andWhereDistance( ):
	 *
	 * @param       boolean   $withDistance :
	 * @return	string    $andWhere : andWhere clause
	 * @access private
	 * @version 6.1.4
	 * @since   0.0.1
	 */
	private function _andWhereDistance( $withDistance )
	{
		if( !$withDistance )
		{
			return null;
		}

		$smallerThanMaxRadius = null;
		if( $this->_andWithMaxRadius() )
		{
			$gp = ( array ) \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( $this->_confGP[ 'parameter' ] );
			$maxRadius = ( int ) $gp[ $this->_confGP[ 'select' ] ];
			$smallerThanMaxRadius = ' < ' . ( $maxRadius + 0 );
		}


		$km = ( double ) $this->_extConf[ 'earth.' ][ 'radius.' ][ 'km' ];
		if( empty( $km ) )
		{
			//  http://de.wikipedia.org/wiki/Erdradius
			$km = 6378.2;
		}

		$destLat = $this->_confFields[ 'lat' ];
		$destLon = $this->_confFields[ 'lon' ];

		$andWhere = ''
						. 'AND ('
						. 'ACOS('
						. 'SIN(RADIANS(tx_radialsearch_postalcodes.latitude)) * SIN(RADIANS(' . $destLat . '))'
						. ' + COS(RADIANS(tx_radialsearch_postalcodes.latitude)) * COS(RADIANS(' . $destLat . '))'
						. ' * COS(RADIANS(tx_radialsearch_postalcodes.longitude) - RADIANS(' . $destLon . '))'
						. ')'
						. ' * ' . $km . ''
						. ') '
						. $smallerThanMaxRadius
		;

		return $andWhere;
	}

	/**
	 * _andWhereEnabledFields( ):
	 *
	 * @return	string
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _andWhereEnabledFields()
	{
		$andWhere = $this->_pObj->cObj->enableFields( 'tx_radialsearch_postalcodes' );
		return $andWhere;
	}

	/**
	 * _andWhereFilter( ):
	 *
	 * @return	string    $andWhere : andWhere clause
	 * @access private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _andWhereFilter()
	{
		$arrAndWhere = array();

		foreach( array_keys( ( array ) $this->_confFilter ) as $filter )
		{
// CONTINUE : filter has an dot
			if( rtrim( $filter, '.' ) != $filter )
			{
				continue;
			}
// CONTINUE : field has an dot

			$name = $this->_confFilter[ $filter ];
			$conf = $this->_confFilter[ $filter . '.' ];
			$value = $this->_pObj->cObj->cObjGetSingle( $name, $conf );

			switch( true )
			{
				case( $value === null ):
				case( $value == '*' ):
				case( $value == '"*"' ):
// do nothing
					break;
				default:
					$arrAndWhere[] = $filter . ' LIKE ' . $value;
					break;
			}
		}

		if( empty( $arrAndWhere ) )
		{
			return null;
		}

		$andWhere = ' AND ' . implode( ' AND ', $arrAndWhere );
		return $andWhere;
	}

	/**
	 * _andWhereSword( ):
	 *
	 * @return	string    $andWhere : andWhere clause
	 * @access private
	 * @version 6.1.4
	 * @since   0.0.1
	 */
	private function _andWhereSword()
	{
		$gp = ( array ) \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( $this->_confGP[ 'parameter' ] );

		$sword = $gp[ $this->_confGP[ 'input' ] ];
// #i0014, dwildt, 2+
		$sword = strip_tags( $sword );
		$sword = htmlspecialchars( $sword );
		$sword = $GLOBALS[ 'TYPO3_DB' ]->quoteStr( $sword, 'tx_radialsearch_postalcodes' );

// #i0018, 160223, dwildt
		$andWhere = ''
						. 'AND'
						. '('
						. 'tx_radialsearch_postalcodes.postal_code LIKE "' . $sword . '%"'
						. ' OR  tx_radialsearch_postalcodes.place_name LIKE "' . $sword . '%"'
						. ' OR  CONCAT(tx_radialsearch_postalcodes.postal_code, " ", tx_radialsearch_postalcodes.place_name) LIKE "' . $sword . '%"'
						. ')'
		;

		return $andWhere;
	}

	/**
	 * _andWithMaxRadius( )  :
	 *
	 * @return	booelan
	 * @internal    #52486
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _andWithMaxRadius()
	{
		if( $this->_andWithMaxRadius !== null )
		{
			return $this->_andWithMaxRadius;
		}

		$searchmode = $this->_confFields[ 'searchmode' ];
		switch( $searchmode )
		{
			case( 'Within the radius only' ):
				$this->_andWithMaxRadius = true;
				break;
			case( 'Within and without the radius' ):
				$this->_andWithMaxRadius = false;
				break;
			default:
				$prompt = 'ERROR: searchmode isn\'t defined!<br />' . PHP_EOL .
								'Sorry for the trouble.<br />' . PHP_EOL .
								'TYPO3 Radial Search<br />' . PHP_EOL .
								__METHOD__ . ' (' . __LINE__ . ')';
				die( $prompt );
				break;
		}

		return $this->_andWithMaxRadius;
	}

	/**
	 * _init( ):
	 *
	 * @return	boolean        true
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _init()
	{
		$this->_initRequirements();
		$this->_initExtConf();
		$this->_initDRS();
		$this->_initSword();
		return true;
	}

	/**
	 * _initDRS( ):
	 *
	 * @return	boolean        true
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _initDRS()
	{
		if( $this->_drs !== null )
		{
			return;
		}

		switch( $this->_extConf[ 'drs.' ][ 'enabled' ] )
		{
			case( 'Disabled' ):
			case( null ):
				$this->_drs = false;
				return;
			case( 'Enabled (for debugging only!)' ):
// Follow the workflow
				break;
			default:
				$prompt = 'Error: drs.enabled is undefined.<br />
          value is ' . $this->_pObj->arr_extConf[ 'drs.' ][ 'enabled' ] . '<br />
          <br />
          ' . __METHOD__ . ' line(' . __LINE__ . ')';
				die( $prompt );
		}

// Set prompt flags
		$this->_drs = true;

// DRS
		$prompt = 'The DRS - Development Reporting System is enabled: ' . $this->_extConf[ 'drs.' ][ 'enabled' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );
		$prompt = 'The DRS is enabled by the extension manager.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * _initExtConf( ):
	 *
	 * @return	boolean        true
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _initExtConf()
	{
		$this->_extConf = unserialize( $GLOBALS[ 'TYPO3_CONF_VARS' ][ 'EXT' ][ 'extConf' ][ 'radialsearch' ] );
		return true;
	}

	/**
	 * _initRequirements( ):
	 *
	 * @return	void
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _initRequirements()
	{
		if( !is_object( $this->_pObj ) )
		{
			$prompt = 'ERROR: no object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Radial Search (Umkreissuche)<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}

		if( !is_array( $this->_confFields ) )
		{
			$prompt = 'ERROR: no array!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Radial Search<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}

		if( !is_array( $this->_confFilter ) )
		{
			$prompt = 'ERROR: no array!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Radial Search<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}
	}

	/**
	 * _initSword( ): Set the class var $isSword
	 *
	 * @return	boolean        true, if sword is set. False, if not.
	 * @access  private
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	private function _initSword()
	{
// RETURN : sword is set before
		if( $this->_isSword !== null )
		{
			return $this->_isSword;
		}
// RETURN : sword is set before
// Get the current sword
		$gp = ( array ) \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( $this->_confGP[ 'parameter' ] );
		$sword = $gp[ $this->_confGP[ 'input' ] ];
// #i0014, dwildt, 2+
		$sword = strip_tags( $sword );
		$sword = htmlspecialchars( $sword );

// Set class var $isSword
		switch( true )
		{
			case( $sword === null ):
			case( $sword == '' ):
			case( $sword == '*' ):
				$this->_isSword = false;
				break;
			default:
				$this->_isSword = true;
				break;
		}
		unset( $sword );
// Set class var $isSword

		return $this->_isSword;
	}

	/**
	 * _setConfigurationDieIfError( )  : 
	 *
	 * @param	array		$fields : array with elements lat and lon
	 * @param	array		$filter : TypoScript configuration of the filter
	 * @return	void
	 * @access private
	 * @version    8.8.1
	 * @since      4.7.0
	 */
	private function _setConfigurationDieIfError( $fields, $filter, $gp )
	{
		if( !is_array( $fields ) )
		{
			$this->_zzDieNoArray( __METHOD__, __LINE__ );
		}

		if( !is_array( $filter ) )
		{
			$this->_zzDieNoArray( __METHOD__, __LINE__ );
		}

		if( empty( $fields[ 'distance' ] ) )
		{
			$this->_zzDieNoFieldDistance( __METHOD__, __LINE__ );
		}

		if( empty( $fields[ 'uid' ] ) )
		{
			$this->_zzDieNoUid( __METHOD__, __LINE__ );
		}

		if( empty( $fields[ 'lat' ] ) )
		{
			$this->_zzDieNoLat( __METHOD__, __LINE__ );
		}

		if( empty( $fields[ 'lon' ] ) )
		{
			$this->_zzDieNoLon( __METHOD__, __LINE__ );
		}

		if( empty( $fields[ 'searchmode' ] ) )
		{
			$this->_zzDieNoSeachmode( __METHOD__, __LINE__ );
		}

		if( !is_array( $gp ) )
		{
			$this->_zzDieNoArray( __METHOD__, __LINE__ );
		}

		return;
	}

	/**
	 * _zzDieNoArray( )  : 
	 *
	 * @param	array		$method : current method
	 * @param	array		$line : current line
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      8.8.1
	 */
	private function _zzDieNoArray( $method, $line )
	{
		$prompt = 'ERROR: no array!<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Radial Search<br />' . PHP_EOL .
						$method . ' (' . $line . ')';
		die( $prompt );
	}

	/**
	 * _zzDieNoFieldDistance( )  : 
	 *
	 * @param	array		$method : current method
	 * @param	array		$line : current line
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      8.8.1
	 */
	private function _zzDieNoFieldDistance( $method, $line )
	{
		$prompt = 'ERROR: field[ distance ] is empty!<br />' . PHP_EOL .
						'Take care of a proper configuration and PHP code.<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Radial Search<br />' . PHP_EOL .
						$method . ' (' . $line . ')';
		die( $prompt );
	}

	/**
	 * _zzDieNoLat( )  : 
	 *
	 * @param	array		$method : current method
	 * @param	array		$line : current line
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      8.8.1
	 */
	private function _zzDieNoLat( $method, $line )
	{
		$prompt = 'ERROR: field[ lat ] is empty!<br />' . PHP_EOL .
						'Take care of a proper configuration and PHP code.<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Radial Search<br />' . PHP_EOL .
						$method . ' (' . $line . ')';
		die( $prompt );
	}

	/**
	 * _zzDieNoLon( )  : 
	 *
	 * @param	array		$method : current method
	 * @param	array		$line : current line
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      8.8.1
	 */
	private function _zzDieNoLon( $method, $line )
	{
		$prompt = 'ERROR: field[ lon ] is empty!<br />' . PHP_EOL .
						'Take care of a proper configuration and PHP code.<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Radial Search<br />' . PHP_EOL .
						$method . ' (' . $line . ')';
		die( $prompt );
	}

	/**
	 * _zzDieNoSeachmode( )  : 
	 *
	 * @param	array		$method : current method
	 * @param	array		$line : current line
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      8.8.1
	 */
	private function _zzDieNoSeachmode( $method, $line )
	{
		$prompt = 'ERROR: field[ searchmode ] is empty!<br />' . PHP_EOL .
						'Take care of a proper configuration and PHP code.<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Radial Search<br />' . PHP_EOL .
						$method . ' (' . $line . ')';
		die( $prompt );
	}

	/**
	 * _zzDieNoUid( )  : 
	 *
	 * @param	array		$method : current method
	 * @param	array		$line : current line
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      8.8.1
	 */
	private function _zzDieNoUid( $method, $line )
	{
		$prompt = 'ERROR: field[ uid ] is empty!<br />' . PHP_EOL .
						'Take care of a proper configuration and PHP code.<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Radial Search<br />' . PHP_EOL .
						$method . ' (' . $line . ')';
		die( $prompt );
	}

	/**
	 * andFrom( ) : Returns the andFrom statement.
	 *              Returns null, if there isn't any sword.
	 *
	 * @return	string    $andFrom : andFrom statement
	 * @access public
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	public function andFrom()
	{
		$this->_init();

		// RETURN : there isn't any sword
		if( !$this->_isSword )
		{
			return null;
		}
		// RETURN : there isn't any sword
		// Set the andFrom statement
		$andFrom = '' .
						' CROSS JOIN  tx_radialsearch_postalcodes ';

		// RETURN : no DRS
		if( !$this->_drs )
		{
			return $andFrom;
		}

		// DRS
		$prompt = $andFrom;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );

		return $andFrom;
	}

	/**
	 * andHaving( ) : Returns the andHaving statement.
	 *              Returns null, if there isn't any sword.
	 *
	 * @return	string    $andHaving : andHaving statement
	 * @access public
	 * @version 6.1.0
	 * @since   0.0.1
	 */
	public function andHaving()
	{
		$this->_init();

		// RETURN : there isn't any sword
		if( !$this->_isSword )
		{
			return null;
		}

		// RETURN : without max radius
		if( !$this->_andWithMaxRadius() )
		{
			$uid = $this->_confFields[ 'uid' ];
			$andHaving = ''
							. ' GROUP BY ' . $uid . ' '
			;
			return $andHaving;
		}

		$distance = $this->_confFields[ 'distance' ];
		$uid = $this->_confFields[ 'uid' ];

		$gp = ( array ) \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( $this->_confGP[ 'parameter' ] );
		$maxRadius = ( int ) $gp[ $this->_confGP[ 'select' ] ];

		// Set the andHaving statement
		$andHaving = ''
						. ' GROUP BY ' . $uid . ' ' // #61797, 150327, dwildt, +
						. ' HAVING ' . $distance . ' < ' . ( $maxRadius + 0 ) . ' '
		;

		// RETURN : no DRS
		if( !$this->_drs )
		{
			return $andHaving;
		}

		// DRS
		$prompt = $andHaving;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );

		return $andHaving;
	}

	/**
	 * andOrderBy( ) : Returns the andOrderBy statement.
	 *              Returns null, if there isn't any sword.
	 *
	 * @return	string    $andOrderBy : andOrderBy statement
	 * @access public
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	public function andOrderBy()
	{
		$this->_init();

		// RETURN : there isn't any sword
		if( !$this->_isSword )
		{
			return null;
		}


		$distance = $this->_confFields[ 'distance' ];

		// Set the andOrderBy statement
		$andOrderBy = '' .
						' ' . $distance . ' ';

		// RETURN : no DRS
		if( !$this->_drs )
		{
			return $andOrderBy;
		}

		// DRS
		$prompt = $andOrderBy;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );

		return $andOrderBy;
	}

	/**
	 * andSelect( ) : Returns the andSelect statement.
	 *                Returns null, if there isn't any sword.
	 *
	 * @return	string    $andSelect : andSelect statement
	 * @access public
	 * @version 6.1.4
	 * @since   0.0.1
	 */
	public function andSelect()
	{
		$this->_init();

		// RETURN : there isn't any sword
		if( !$this->_isSword )
		{
			return null;
		}

		$km = ( double ) $this->_extConf[ 'earth.' ][ 'radius.' ][ 'km' ];
		if( empty( $km ) )
		{
			//  http://de.wikipedia.org/wiki/Erdradius
			$km = 6378.2;
		}

		$distance = $this->_confFields[ 'distance' ];
		$destLat = $this->_confFields[ 'lat' ];
		$destLon = $this->_confFields[ 'lon' ];

		$andSelect = ''
						. ', '
						. 'ACOS('
						. 'SIN(RADIANS(tx_radialsearch_postalcodes.latitude)) * SIN(RADIANS(' . $destLat . '))'
						. ' + COS( RADIANS(tx_radialsearch_postalcodes.latitude)) * COS(RADIANS(' . $destLat . '))'
						. ' * COS( RADIANS(tx_radialsearch_postalcodes.longitude) - RADIANS(' . $destLon . '))'
						. ')'
						. ' * ' . $km . ' AS \'' . $distance . '\''
		;

		// RETURN : no DRS
		if( !$this->_drs )
		{
			return $andSelect;
		}

		// DRS
		$prompt = $andSelect;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );

		return $andSelect;
	}

	/**
	 * andWhere( )  : Returns the andWhere clause.
	 *                Returns null, if there isn't set any sword.
	 *
	 * @param       boolean   $withDistance :
	 * @return	string    $andWhere     : andWhere clause
	 * @access public
	 * @version 0.0.1
	 * @since   0.0.1
	 */
	public function andWhere( $withDistance = false )
	{
		$this->_init();

		// RETURN : there isn't any sword
		if( !$this->_isSword )
		{
			return null;
		}

		// Get the pid
		$pid = ( int ) $this->_extConf[ 'database.' ][ 'pid' ];

		// Set the andWhere statement
		$andWhere = '' .
						' AND tx_radialsearch_postalcodes.pid = ' . $pid . PHP_EOL
						. $this->_andWhereFilter() . PHP_EOL
						. $this->_andWhereSword() . PHP_EOL
						. $this->_andWhereEnabledFields() . PHP_EOL
						. $this->_andWhereDistance( $withDistance ) . PHP_EOL
		;

		// RETURN : no DRS
		if( !$this->_drs )
		{
			return $andWhere;
		}

		// DRS
		$prompt = $andWhere;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->extKey, 0 );

		return $andWhere;
	}

	/**
	 * getFieldForLatLon( ) : Returns an array with the field labels of the latitude and the longitude
	 *
	 * @return	array         Returns an array with the field labels of the latitude and the longitude
	 * @access public
	 * @internal #i0012
	 * @version    6.0.2
	 * @since      6.0.2
	 */
	public function getFieldForLatLon()
	{
		if( !empty( $this->_confFields ) )
		{
			return array(
					'lat' => $this->_confFields[ 'lat' ],
					'lon' => $this->_confFields[ 'lon' ]
			);
		}

		// Die in case of an error
		$arrPrompt = array(
				0 => '',
				1 => '$this->_confFields is empty',
				2 => '',
				3 => '',
				4 => 'Method: ' . __METHOD__ . ' #' . __LINE__,
				5 => 'TYPO3 extension Radial Search',
		);
		$prompt = implode( '<br />' . PHP_EOL, $arrPrompt );

		die( $prompt );
	}

	/**
	 * setConfiguration( )  : Set fields and filter
	 *
	 * @param	array		$fields : array with elements lat and lon
	 * @param	array		$filter : TypoScript configuration of the filter
	 * @return	void
	 * @access public
	 * @version    8.8.1
	 * @since      4.7.0
	 */
	public function setConfiguration( $fields, $filter, $gp )
	{
		$this->_setConfigurationDieIfError( $fields, $filter, $gp );

		$this->_confFields = $fields;
		$this->_confFilter = $filter;
		$this->_confGP = $gp;

		return true;
	}

	/**
	 * setParentObject( )  : Set the parent object
	 *
	 * @param	object		$pObj: Parent Object
	 * @return	void
	 * @access public
	 * @version    0.0.1
	 * @since      0.0.1
	 */
	public function setParentObject( $pObj )
	{
		if( !is_object( $pObj ) )
		{
			$prompt = 'ERROR: no object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Radial Search<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}
		$this->_pObj = $pObj;
	}

}

if( defined( 'TYPO3_MODE' ) && $TYPO3_CONF_VARS[ TYPO3_MODE ][ 'XCLASS' ][ 'ext/radialsearch/Classes/Gateway/SqlParts.php' ] )
{
	include_once($TYPO3_CONF_VARS[ TYPO3_MODE ][ 'XCLASS' ][ 'ext/radialsearch/Classes/Gateway/SqlParts.php' ]);
}
?>